// #include <math.h>
// #include <stdio.h>
// #include <stdlib.h>
#include <malloc.h>
#include <mpi.h>
#include "header.h"

void JacobiSmooth(const int nx, const int ny, const int neigh[], const int color,
		  const int myid, const MPI::Intracomm& icomm,
		  const double sk[], const int id[], const int ik[],
		  const double f[], double u[], const double dd[],
		  double aux[], const int maxiter )
{
  const double omega = 0.8;
  const int nnode = (nx+1)*(ny+1);
  double *r = aux;                            /* ( and  w = r ! )   */

  /* Iteration sweeps */                      /* no inner product ! */
  int iter  = 0;
  while ( maxiter>iter)
  {
    iter++;
    
    vdcopy(nnode, r, 1, f, 1);			/*                 */
    CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);/*  r := f - K*u   */

    vddiv(nnode, r, 1, r, 1, dd, 1);		/*  w := D^{-1}*r  */ 
    VecAccu(nx,ny,r,neigh,color,icomm);	/*  dist.-->acc.   */
    
    vdaxpy(nnode, u, u, omega, r );		/*  u := u + om*w  */  
    
/*    if (myid==0)  printf("Iteration %u : %g\n",iter);     */
  }
  
  return;
}


void JacobiSolveGlob(const int myid, const MPI::Intracomm& icomm,
		     const int nloc, const int nglob,
		     const double csk[], const int cid[], const int cik[],
		     const int l2g[], const double f[],  double u[] )
{
  const double dzero=0.0, omega=1.0, tol=1e-5, tol2=tol*tol;
  const int maxiter = 100;
  double *dd, *cr, *cf, *cu, sigma, sigma0;
  int    iter, i, ia;

  cf    = (double *) malloc(nglob*sizeof(double));	/* r.h.s.   */
  cr    = (double *) calloc(nglob,sizeof(double));	/* residual */

  for (i=0; i<nloc; i++)  cr[l2g[i]] = f[i];

//   MPI_Allreduce(cr, cf, nglob, MPI_DOUBLE, MPI_SUM, icomm);
  icomm.Allreduce(cr, cf, nglob, MPI_DOUBLE, MPI_SUM);

  dd    = (double *) malloc(nglob*sizeof(double));	/* matrix diagonal */
  ia = 0;
  for (i = 0; i < nglob; i++)
    { dd[i] = csk[ia];  ia = cid[i]+1; }

  /*  Choose initial guess = 0 */
  cu    = (double *) calloc(nglob,sizeof(double));	/*  u := 0  */

  /*  Initial sweep  */
  vdcopy(nglob, cr, 1, cf, 1);				/*                 */
  CrsMult(0, nglob-1, cr, cu, cid, cik, csk, -1.0);	/*  r := f - K*u   */

  vddiv(nglob, cr, 1, cr, 1, dd, 1);		/*  r := D^{-1}*r  */
  sigma0 = dscapr(nglob,cr,cr);			/* s0 := <r,r>     */

  /* Iteration sweeps */
  iter  = 0;
  sigma = sigma0;
  while ( sigma > tol2*sigma0 && maxiter>iter)
  {
    iter++;
    vdaxpy(nglob, cu, cu, omega, cr );		/*  u := u + om*r  */
    vdcopy(nglob, cr, 1, cf, 1);		/*                 */
    CrsMult(0, nglob-1, cr, cu, cid, cik, csk, -1.0);/*  r := f - K*u   */

    vddiv(nglob, cr, 1, cr, 1, dd, 1);		/*  r := D^{-1}*r  */
    sigma = dscapr(nglob,cr,cr);		/* s0 := <r,r>     */

/*    if (myid==0)
    	printf("Iteration %u : %g\n",iter,sqrt(sigma/sigma0));    */
  }
/*
  if (myid==0)
  	printf("aver. Jacobi rate :  %g\n", exp(log(sqrt(sigma/sigma0))/iter));
*/
  for (i=0; i<nloc; i++)  u[i] = cu[l2g[i]];

  free(cu); free(dd); free(cr); free(cf);
  return;
}
