//  Compile with  
//   make 2>&1 | grep -v openmpi
//  to avoid warnings caused by OpenMPI

#include <iostream>
#include <cmath>
#include <cstdlib>
#include <mpi.h>
using namespace std;
#include "header.h"
#include "mg.h"

int main(int argc, char* argv[])
{
//     MPI_Comm icomm;
//     int   nx, ny, nnx, nny, *id, *ik, nnode, sksize, ScaleUp,
//       i, zero, procx, procy, numprocs, myid, NXglob, NYglob,
//       neigh[4], color, nlevels, level, ierr;
//     double xl, xr, yb, yt, *sk, dzero=0., *u, *f,
//       omega,  done = 1.,
//       t_start, t_mgm, usertime();
//     ControlStruct control;
//     PointerStruct pointers;
    const bool bdebug=false;		// debugging at the end??
    const int nlevels = 3; 				/* set number of mg levels */

    MPI::Init(argc,argv);
    MPI::Intracomm icomm(MPI::COMM_WORLD);
    const int numprocs = icomm.Get_size();
    const int myrank   = icomm.Get_rank();

    if (myrank == 0)
	    { cout << "\n There are " << numprocs << " processes running.\n \n"; }
    icomm.Barrier();

    const int procx = static_cast<int>(sqrt(numprocs+0.0));
    const int procy = procx;
// 

    if (procy * procx != numprocs)
    {
      cout << "\n Wrong number of processors !\n \n";
    }
    else
      {
      int neigh[4], color;
      IniGeom(myrank, procx, procy, neigh, color);

      icomm.Barrier();
      /*                                              subdomain coordinates */
      double xl,xr,yb,yt;
      IniCoord(myrank, procx, procy, xl, xr, yb, yt);


      bool ScaleUp = !true;
      int nx,ny,NXglob,NYglob;
      /* number of local intervals on (xl,xr)=:nx, (yb,yt)=:ny */
      if (ScaleUp)
        {
      /*	 	for scale up evaluation   */
        if (myrank==0) cout << "scale up evaluation\n";
	nx = 2;
	ny = 2;
	NXglob = nx*procx;
	NYglob = ny*procy;
        }
      else
        {
      /*		for speed up evaluation   */
        if (myrank==0) cout << "speed up evaluation\n";
	NXglob = NYglob = 12;
	nx = NXglob/procx;
	ny = NYglob/procy;
	if ( NXglob % procx != 0   || NYglob % procy != 0 )
	   { MPI_Finalize();
	     cout << "\n Check discretization vs. number of processors\n";
	     exit(-1);
	   }
        }

// 	nlevels -= (int)(log((double)procx)/log(2.0));
//      }
      if (myrank==0)
      {
//	 printf("%u level on %u x %u processors. \n",nlevels,procx,procy);
         cout << nlevels << " levels on  " << procx << " x " << procy << " processors. \n";
      }
/*      			t_start = usertime();	*/
      double t_start = MPI_Wtime(), t_mgm;
      int ierr;
      ControlStruct control;
      PointerStruct pointers;
      /* Set mg control parameters */
      control.cycle = 1;		/* V-cycle			*/
      control.npre  = 1;		/* #sweeps in presmoothing	*/
      control.npost = 1;		/* #sweeps in postsmoothing	*/

 // C --> C++
      ierr = AllocLevels(nlevels, nx, ny, &pointers);
      if (ierr!=0) printf("\nError No. %i in AllocLevels.\n",ierr);
      const int n1 = pointers.nx[nlevels-1]+1;
      if (myrank==0) printf("Unknowns: (%u)^2 = %d per processor\n",n1,n1*n1);

      ierr = IniLevels(xl, xr, yb, yt, neigh, color, myrank, icomm, nlevels, &pointers);

      ierr = GlobalCoarse(neigh, color, myrank, icomm, nlevels, &pointers);
//
      MGMSolver(&pointers, &control, nlevels-1, neigh,color,myrank,icomm);
//
      icomm.Barrier();
//
// /*      			t_mgm = usertime()-t_start;	*/
// 			t_mgm = MPI_Wtime()-t_start;
//
      const int level = nlevels-1;
//
      SaveVectorP(myrank, "t.dat", pointers.u[level], pointers.nx[level],
      				pointers.ny[level], xl, xr, yb, yt);
      if (myrank==0) printf("-----------\n");
//
      FreeVecAccu();
      ierr = FreeLevels(nlevels, &pointers);

  }
  icomm.Barrier();

  //			if (myid==0) printf("End of program: %g sec.\n",t_mgm);
  MPI::Finalize();
  return 0;
}


