
#
#	MPI: jacobi
#				Gundolf Haase, Johannes Kepler University Linz
#				ghaase@numa.uni-linz.ac.at
#
MACHINE=LINUX
PROGRAM= mgm.$(MACHINE)

objects=main.o vdop.o  dexx.o debugd.o skalar.o\
	geom.o addbound.o getbound.o vecaccu.o\
	getmatrix.o applybc.o crsmult.o userset.o\
	accudiag.o jacsolve.o savevekp.o\
	applybc1.o allocati.o gridtransfer.o jacsmooth.o mgm.o

PROGRAM2=try
objects2=try.o mesh.o group.o
# commsets.o


default: $(PROGRAM)

include ../default.mk


$(PROGRAM2):: $(objects2)
	$(CXX) $(LDFLAGS) -o $(PROGRAM2) $(objects2) $(LDFLAGS)

$(PROGRAM):: $(objects)
	$(CXX) $(LDFLAGS) -o $(PROGRAM) $(objects) $(LDFLAGS)
#
	@echo
	@echo "Start with :  $(MPIRUN) -np num_proc $(MPIFLAGS) $(PROGRAM)"
	@echo

test: $(PROGRAM)
	$(MPIBOOT)
#	$(MPIBOOT) -v $(MPINODES)
	$(MPIRUN) -np 4 $(MPIFLAGS) $(PROGRAM)
	$(MPICLEAN)

TAR_INPUT=try.cpp mesh.* group.* *.txt Makefile
#commsets.*
TAR_FILE=try.tar

tar:
	tar cf $(TAR_FILE) $(TAR_INPUT)


NrProcesses=1 4 16
# 9 25 36
eval: $(PROGRAM)
	@for i in $(NrProcesses);  \
	do  echo $$i " processors"; \
	    $(MPIRUN) $(MPIFLAGS) -np $$i $(PROGRAM) > out.$$i; \
	    $(MPICLEAN); \
	done


NPROCS	= 4

TFILE	= t.dat
TTMP	= t.tmp

graph: $(PROGRAM)
	@rm -f $(TFILE).*
	$(MPIRUN) $(MPIFLAGS) -np $(NPROCS) $(PROGRAM)
	@echo " "; echo "Manipulate data for graphics."; echo " "
	@cat $(TFILE).* > $(TTMP)
	@sort -b -k 2    $(TTMP)   -o $(TTMP).1
	@sort -b -k 1    $(TTMP).1 -o $(TTMP).2
	@awk  -f nl.awk  $(TTMP).2  > $(TFILE)
	@rm -f $(TTMP).* $(TTMP) $(TFILE).*

	-gnuplot mgm.dem




mem: ${PROGRAM}
	valgrind -v --leak-check=yes --tool=memcheck --undef-value-errors=yes --track-origins=yes --log-file=$^.addr.out --show-reachable=yes $(MPIRUN) $(MPIFLAGS) -np 4 ./$^














